# -*- coding: utf-8 -*-
"""Imports for the Windows Registry parser."""

from plaso.parsers.winreg_plugins import appcompatcache
from plaso.parsers.winreg_plugins import bagmru
from plaso.parsers.winreg_plugins import ccleaner
from plaso.parsers.winreg_plugins import default
from plaso.parsers.winreg_plugins import lfu
from plaso.parsers.winreg_plugins import mountpoints
from plaso.parsers.winreg_plugins import mrulist
from plaso.parsers.winreg_plugins import mrulistex
from plaso.parsers.winreg_plugins import msie_zones
from plaso.parsers.winreg_plugins import network_drives
from plaso.parsers.winreg_plugins import officemru
from plaso.parsers.winreg_plugins import outlook
from plaso.parsers.winreg_plugins import networks
from plaso.parsers.winreg_plugins import programscache
from plaso.parsers.winreg_plugins import run
from plaso.parsers.winreg_plugins import sam_users
from plaso.parsers.winreg_plugins import services
from plaso.parsers.winreg_plugins import shutdown
from plaso.parsers.winreg_plugins import task_scheduler
from plaso.parsers.winreg_plugins import terminal_server
from plaso.parsers.winreg_plugins import typedurls
from plaso.parsers.winreg_plugins import timezone
from plaso.parsers.winreg_plugins import userassist
from plaso.parsers.winreg_plugins import usb
from plaso.parsers.winreg_plugins import usbstor
from plaso.parsers.winreg_plugins import winlogon
from plaso.parsers.winreg_plugins import winrar
from plaso.parsers.winreg_plugins import windows_version
